#! /usr/bin/env python3
import csv
from datetime import datetime
from pprint import pprint

import mt940

from schmeckels import models

SUPPORTED_BANKS = (
    ("dkb", "DKB"),
    ("sparkasse-mt940", "Sparkasse MT940"),
    ("bunq-csv", "Bunq CSV"),
)


class Bank(object):
    """
    A class to represent a bank

    ...

    Attributes
    ----------
    filename : str
        path to the file containing the transactions

    Methods
    -------
    get_transactions()
        Returns a list of all transactions
    """

    def __init__(self, filename):
        self.filename = filename

    def get_transactions(self):
        """
        Returns a Transction model for every transction in the file
        """
        pass


class Sparkasse_MT940(Bank):
    """
    Handle import for MT940 files from Sparkasse
    """

    def get_transactions(self):
        print(self.filename)
        transactions = mt940.parse(self.filename)

        for t in transactions:
            data = t.data
            date = data["date"]
            amount = int(data["amount"].amount * 100)
            iban = data.get("applicant_iban", "")
            name = data.get("applicant_name", "")
            description = data["purpose"]

            yield models.Transaction(date=date, name=name, iban=iban, amount=amount, description=description)


class Bunq(Bank):
    """
    Handle import for CSV exports of bunq
    """

    def get_transactions(self):
        with open(self.filename) as fh:
            fh.readline()  # Get rid of first line
            csv_reader = csv.reader(fh, delimiter=";")
            for line in csv_reader:
                date = datetime.strptime(line[0], "%Y-%m-%d")
                amount = int(float(line[2].replace(",", "")) * 100)
                iban = line[4]
                name = line[5]
                description = line[6]

                yield models.Transaction(date=date, name=name, iban=iban, amount=amount, description=description)


class DKB(Bank):
    """
    Handle import for CSV from DKB
    """

    def get_transactions(self):
        with open(self.filename, encoding="ISO-8859-1") as fh:
            csv_reader = csv.reader(fh, delimiter=";")
            next(csv_reader, None)
            for line in csv_reader:
                date = datetime.strptime(line[0], "%d.%m.%Y")
                amount = int(line[7].replace(".", "").replace(",", ""))
                iban = line[5]
                name = line[3]
                description = line[4]
                yield models.Transaction(date=date, name=name, iban=iban, amount=amount, description=description)
