#! /usr/bin/env python3
import calendar
from datetime import date, datetime
from math import floor

import click
from dateutil.relativedelta import relativedelta
from flask import Flask, flash, redirect, render_template, request
from sqlalchemy import and_, func

from schmeckels import banks
from schmeckels.helper import format_amount, get_session, import_transactions
from schmeckels.models import Tag, Transaction

session = None
app = Flask(__name__)


@app.route("/")
def index():
    tag_sum_in = {}
    tag_sum_out = {}

    # Get a list of all tags
    tags = session.query(Tag).filter_by(reporting=True).all()

    # Get start and end of timerange
    months = 3
    today = datetime.now()
    start_month = today - relativedelta(months=months)
    start_date = date(start_month.year, start_month.month, 1)
    end_date = today

    for tag in tags:
        tag_sum = (
            session.query(func.sum(Transaction.amount))
            .filter(and_(Transaction.date >= start_date, Transaction.date <= end_date))
            .filter(Transaction.tags.any(id=tag.id))
            .first()[0]
        )
        if tag_sum:
            if tag_sum >= 0:
                tag_sum_in[tag] = {"sum": tag_sum, "monthly": floor(tag_sum / months)}
            else:
                tag_sum_out[tag] = {"sum": tag_sum, "monthly": floor(tag_sum / months)}

    monthly_sum = {
        "in": format_amount(sum([v["monthly"] for k, v in tag_sum_in.items()])),
        "out": format_amount(sum([v["monthly"] for k, v in tag_sum_out.items()])),
    }
    tag_sum_in = {
        k: format_amount(v) for k, v in sorted(tag_sum_in.items(), key=lambda item: float(item[1]["sum"]), reverse=True)
    }
    tag_sum_out = {k: format_amount(v) for k, v in sorted(tag_sum_out.items(), key=lambda item: float(item[1]["sum"]))}

    return render_template(
        "index.html", tag_sum_in=tag_sum_in, tag_sum_out=tag_sum_out, months=months, monthly_sum=monthly_sum
    )


@app.route("/tags")
def tags():
    tags = session.query(Tag).all()

    # Sort alphabetically
    tags = sorted(tags, key=lambda item: item.name)
    return render_template("tags.html", tags=tags)


@app.route("/tag/<name>")
def tag(name):
    tag = session.query(Tag).filter(Tag.name == name).first()
    transactions = session.query(Transaction).filter(Transaction.tags.any(id=tag.id)).all()
    sum_months = (
        session.query(
            func.sum(Transaction.amount).label("amount"), func.strftime("%Y-%m", Transaction.date).label("month")
        )
        .filter(Transaction.tags.any(id=tag.id))
        .group_by("month")
        .all()
    )
    months = []
    for m in sum_months:
        months.append({"amount": format_amount(m["amount"]), "month": m["month"]})

    return render_template("tag.html", tag=tag, months=months, transactions=transactions)


@app.route("/tag/<name>/delete")
def delete_tag(name):
    x = session.query(Tag).filter(Tag.name == name).first()
    if x:
        session.delete(x)
        session.commit()

    return redirect("/tags")


@app.route("/tag/<name>/toggle")
def toggle_tag(name):
    x = session.query(Tag).filter(Tag.name == name).first()
    if x:
        x.toggle_reporting()
        session.add(x)
        session.commit()

    return redirect("/tags")


@app.route("/transactions")
def transactions():
    transactions = session.query(Transaction)

    if request.args.get("unsorted", None):
        transactions = transactions.filter(Transaction.tags == None)

    transactions = transactions.all()

    tags = session.query(Tag).all()
    return render_template("transactions.html", transactions=transactions, tags=tags)


@app.route("/bulksort", methods=["POST"])
def buksort():
    tag_id = request.form.get("tag")
    tag = session.query(Tag).get(tag_id)
    transaction_ids = request.form.getlist("transaction")

    for id in transaction_ids:
        t = session.query(Transaction).get(id)
        t.tags.append(tag)
        session.add(t)
    session.commit()

    return redirect("/transactions")


@app.route("/upload", methods=["GET", "POST"])
def upload():
    global session

    if request.method == "POST":
        file = request.files.get("file")
        if not file or file.filename == "":
            flash("No file choosen")
            return redirect(request.url)

        path = f"/tmp/{file.filename}"
        file.save(path)
        new = import_transactions(session, request.form["bank"], path)
        if len(new) > 0:
            session.bulk_save_objects(new)
            session.commit()
        flash(f"Imported {len(new)} new transactions")
        return render_template("upload.html", banks=banks.SUPPORTED_BANKS, transactions=new)

    return render_template("upload.html", banks=banks.SUPPORTED_BANKS)


@click.command(name="serve")
def command():
    global session
    session = get_session()
    app.secret_key = "WeDon'tHaveSessionsSoThisMustNotBeVerySecure!"
    app.run(host="0.0.0.0", port=8080)
